@@ -1,4 +1,5 @@
 // Copyright (c) 2015-2017 Josh Blum
+// Copyright (c) 2018 Deepwave Digital, Inc.
 // SPDX-License-Identifier: GPL-3.0
 
 #ifdef UHD_HAS_SET_PUBLISHER
@@ -36,6 +37,8 @@
 #include <boost/weak_ptr.hpp>
 #include <boost/algorithm/string.hpp>
 
+#include <algorithm>
+
 //Report a positive gain step value for UHD's automatic distribution algorithm.
 //This prevents the gain group rounding algorithm from producing zero values.
 static const double MIN_GAIN_STEP = 0.1;
@@ -151,7 +154,7 @@ public:
         if (stream) stream->issue_stream_cmd(cmd);
     }
 
-    void setupChannelHooks(const int dir);
+    void setupChannelHooks();
     void setupChannelHooks(const int dir, const size_t chan, const std::string &dirName, const std::string &chName);
     void setupFakeChannelHooks(const int dir, const size_t chan, const std::string &dirName, const std::string &chName);
 
@@ -276,8 +279,7 @@ UHDSoapyDevice::UHDSoapyDevice(const uhd::device_addr_t &args)
     }
 
     //setup channel and frontend hooks
-    this->setupChannelHooks(SOAPY_SDR_RX);
-    this->setupChannelHooks(SOAPY_SDR_TX);
+    this->setupChannelHooks();
 }
 
 UHDSoapyDevice::~UHDSoapyDevice(void)
@@ -286,17 +288,31 @@ UHDSoapyDevice::~UHDSoapyDevice(void)
     SoapySDR::Device::unmake(_device);
 }
 
-void UHDSoapyDevice::setupChannelHooks(const int dir)
+void UHDSoapyDevice::setupChannelHooks()
 {
-    const std::string dirName((dir==SOAPY_SDR_RX)?"rx":"tx");
-    for (size_t ch = 0; ch < _device->getNumChannels(dir); ch++)
+    static const std::string kRxDirName = "rx";
+    static const std::string kTxDirName = "tx";
+    const size_t numRxChannels = _device->getNumChannels(SOAPY_SDR_RX);
+    const size_t numTxChannels = _device->getNumChannels(SOAPY_SDR_TX);
+
+    //We have to build up the same number of TX and RX channels to make UHD
+    //happy. If there are less channels in one direction than another, we fill
+    //in the direction with dummy channels.
+    const size_t numChannels = std::max(numRxChannels, numTxChannels);
+
+    for (size_t ch = 0; ch < numChannels; ch++)
     {
         const std::string chName(boost::lexical_cast<std::string>(ch));
-        this->setupChannelHooks(dir, ch, dirName, chName);
-    }
+        if (ch < numRxChannels)
+            this->setupChannelHooks(SOAPY_SDR_RX, ch, kRxDirName, chName);
+        else
+            this->setupFakeChannelHooks(SOAPY_SDR_RX, ch, kRxDirName, chName);
 
-    //cant have a completely empty direction for apps to work
-    if (_device->getNumChannels(dir) == 0) this->setupFakeChannelHooks(dir, 0, dirName, "0");
+        if (ch < numTxChannels)
+            this->setupChannelHooks(SOAPY_SDR_TX, ch, kTxDirName, chName);
+        else
+            this->setupFakeChannelHooks(SOAPY_SDR_TX, ch, kTxDirName, chName);
+    }
 }
 
 void UHDSoapyDevice::setupChannelHooks(const int dir, const size_t chan, const std::string &dirName, const std::string &chName)
@@ -315,7 +331,7 @@ void UHDSoapyDevice::setupChannelHooks(const int dir, const size_t chan, const s
     //names of the tunable components
     const std::vector<std::string> comps = _device->listFrequencies(dir, chan);
     const std::string rfCompName = (comps.size()>0)?comps.at(0):"RF";
-    const std::string bbCompName = (comps.size()>1)?comps.at(1):"BB";
+    const std::string bbCompName = (comps.size()>1)?comps.at(1):"";
 
     //samp rate
     _tree->create<uhd::meta_range_t>(dsp_path / "rate" / "range")
@@ -324,12 +340,21 @@ void UHDSoapyDevice::setupChannelHooks(const int dir, const size_t chan, const s
         .publish(boost::bind(&SoapySDR::Device::getSampleRate, _device, dir, chan))
         .subscribe(boost::bind(&UHDSoapyDevice::set_sample_rate, this, dir, chan, _1));
 
+    //dsp freq (when there is no tunable cordic)
+    if (bbCompName.empty())
+    {
+        _tree->create<double>(dsp_path / "freq" / "value").set(0.0);
+        _tree->create<uhd::meta_range_t>(dsp_path / "freq" / "range").set(uhd::meta_range_t(0.0, 0.0));
+    }
     //dsp freq
-    _tree->create<double>(dsp_path / "freq" / "value")
-        .publish(boost::bind(&SoapySDR::Device::getFrequency, _device, dir, chan, bbCompName))
-        .subscribe(boost::bind(&UHDSoapyDevice::set_frequency, this, dir, chan, bbCompName, _1));
-    _tree->create<uhd::meta_range_t>(dsp_path / "freq" / "range")
-        .publish(boost::bind(&UHDSoapyDevice::get_freq_range, this, dir, chan, bbCompName));
+    else
+    {
+        _tree->create<double>(dsp_path / "freq" / "value")
+            .publish(boost::bind(&SoapySDR::Device::getFrequency, _device, dir, chan, bbCompName))
+            .subscribe(boost::bind(&UHDSoapyDevice::set_frequency, this, dir, chan, bbCompName, _1));
+        _tree->create<uhd::meta_range_t>(dsp_path / "freq" / "range")
+            .publish(boost::bind(&UHDSoapyDevice::get_freq_range, this, dir, chan, bbCompName));
+    }
 
     //old style stream cmd
     if (dir == SOAPY_SDR_RX)
